#include "stdafx.h"
#include "clf_random.h"
#include "clf_primitives.h"
#include "clf_geometry.h"
#include "clf_camera.h"
#include "clf_resources.h"
#include "clf_light.h"
#include "clf_font.h"

extern HWND  hWnd;						// Window Handle
extern bool keyDown[256];			// holds true / false for each key

static int m_nFPS = 0;
static int m_nFrames = 0;											// FPS and FPS Counter

cCamera g_cam;
clf_stage g_stage;

// a light source
clf_light g_light(GL_LIGHT0);

float g_rotation = 0.f;

// this function is called when the program starts to initialize the game
void game_init()
{
	get_font_manager().create_font_2d("Bookman2D", "Bookman Old Style", 24);
	get_font_manager().create_font_3d("Bookman3D", "Bookman Old Style", 20, .2f);

	g_cam.setup(clf_vector3D(3,3,-10),       // position viewer
							clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
							clf_vector3D(0,1,0));       // which way is up

	glEnable (GL_DEPTH_TEST);        // enable depth test
	glEnable(GL_COLOR_MATERIAL);     // use color for drawing the object

	// create and position a light source
	g_light.set_ambient(0.0f, 0.0f, 0.0f, 1.f);
	g_light.set_diffuse(0.9f, 0.9f, 0.9f, 1.f);
	g_light.set_specular(1.f, 1.f, 1.f, 1.f);
	g_light.set_position(-300.f, 100.f, -300.f, 0.f);
}

// this function is called when the window is resized
void resize_window (int width, int height)			// reshape the window when it's moved or resized
{
	g_stage.resize(0,0,width, height);
	//g_stage.resize(width/4, height/4,width/2, height/2);

	glMatrixMode (GL_PROJECTION);									// select the projection matrix
	glLoadIdentity ();														// reset the projection matrix

	// viewport and perspective
	g_stage.prepare(); 
}

// this function is called upon exit to clear allocated resources
void game_terminate()
{
}

// the core of the game. This is called repeatedly to update the state of the game
// fElapsed: is the elapsed time since last call. The time is in seconds (usualy a very small number)
void frame_move(float fElapsed)
{
	if (keyDown[VK_ESCAPE]) 
		DestroyWindow(hWnd);

	g_rotation += 45.f*fElapsed;

	// frame counting mechanism
	// the higher the frame rate the faster the system
	// if frame rate drops below 30 we are in deep trouble
	// we should either optimize the program or buy a new computer
	static dFloat tot = 0;
	tot += fElapsed;
	m_nFrames++;												// Increment Our frame Counter
	if (tot >= 1.f)
	{
		tot = 0;
		m_nFPS = m_nFrames;
		m_nFrames = 0;

		char txt[200];
		sprintf(txt, "GusOnGames, fps:%d",m_nFPS);
		SetWindowText(hWnd, txt);
	}
}

// the drawing function
// it is called right after the frame_move function
void frame_render()
{
	// clear the screen
	glClearColor (0.f, 0.f, 0.2f, 1.f);				// background colour
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// clear screen and depth buffer

	glMatrixMode (GL_MODELVIEW);								// select the modelview matrix
	glLoadIdentity ();													// reset

	// position the viewer
	g_cam.apply();
	
	// position the light
	g_light.enable();
	g_light.apply();

	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glDisable(GL_TEXTURE_2D);

	clf_font* menu_font = get_font_manager().get_font("Bookman3D");
	menu_font->set_color_RGB(0.5f,.75f,.95f);
	glPushMatrix();
		glTranslatef(1, 0, 0);
		glRotatef(g_rotation, 0,1,0);
		menu_font->print_3d(ALIGN_CENTER, "clf_font3D");
	glPopMatrix();

	clf_font* gfnt = get_font_manager().get_font("Bookman2D");

	// comment or uncomment the folloing line to activate the two different positioning modes
#define FONT_USE_SCRFEEN

#ifdef FONT_USE_SCRFEEN
	gfnt->use_screen_position(true);
	gfnt->set_position_2d(10, 10);
#else
	gfnt->use_screen_position(false);
	glRasterPos2f(-2, -2);
#endif

	gfnt->set_color_RGB(1.f,1.f,1.f);
	gfnt->print_2d("fps:%d",m_nFPS);
	glPopAttrib();
}
